/*
 * Copyright (C) 2012 CERN (www.cern.ch)
 * Author: Aurelio Colosimo
 *
 * Released to the public domain
 */

#include <stdint.h>
#include <ppsi/ppsi.h>
#include <pps_gen.h>
#include <softpll_ng.h>
#include "../proto-ext-whiterabbit/wr-constants.h"
#include <rxts_calibrator.h>
#include "wrpc.h"

extern uint32_t cal_phase_transition[];

int wrpc_spll_locking_enable(struct pp_instance *ppi)
{
	int port = atoi(&ppi->iface_name[2]);
	spll_init(SPLL_MODE_SLAVE, port, 1);
	spll_enable_ptracker(port, 1);
	rxts_calibration_start(port);
	return WR_SPLL_OK;
}

int wrpc_spll_locking_poll(struct pp_instance *ppi, int grandmaster)
{
	int locked;
	int i;
	static int t24p_calibrated[wr_num_ports]={0,0};
	int port = atoi(&ppi->iface_name[2]);

	locked = spll_check_lock(0); /* both slave and gm mode */

	if (grandmaster)
		return locked ? WR_SPLL_READY : WR_SPLL_ERROR;

	/* Else, slave: ensure calibration is done */
	if(!locked) {
		for(i = 0; i<wr_num_ports; ++i)
			t24p_calibrated[i] = 0;
	} else if(locked && !(t24p_calibrated[port])) {
		/*run t24p calibration if needed*/
		if (calib_t24p(WRC_MODE_SLAVE, &cal_phase_transition[port], port) < 0)
			return WR_SPLL_CALIB_NOT_READY;
		t24p_calibrated[port] = 1;
	}

	return locked ? WR_SPLL_READY : WR_SPLL_ERROR;
}

int wrpc_spll_locking_disable(struct pp_instance *ppi)
{
	/* softpll_disable(); */
	return WR_SPLL_OK;
}

int wrpc_spll_enable_ptracker(struct pp_instance *ppi)
{
	spll_enable_ptracker(atoi(&ppi->iface_name[2]), 1);
	return WR_SPLL_OK;
}

int wrpc_enable_timing_output(struct pp_instance *ppi, int enable)
{
	if (enable == WR_DSPOR(ppi)->ppsOutputOn)
		return WR_SPLL_OK;
	WR_DSPOR(ppi)->ppsOutputOn = enable;

	shw_pps_gen_enable_output(enable);
	shw_pps_gen_time_valid(enable);
	return WR_SPLL_OK;
}

int wrpc_adjust_in_progress(void)
{
	return shw_pps_gen_busy() || spll_shifter_busy(0);
}

int wrpc_adjust_counters(int64_t adjust_sec, int32_t adjust_nsec)
{
	if (adjust_sec)
		shw_pps_gen_adjust(PPSG_ADJUST_SEC, adjust_sec);
	if (adjust_nsec)
		shw_pps_gen_adjust(PPSG_ADJUST_NSEC, adjust_nsec);
	return 0;
}

int wrpc_adjust_phase(int32_t phase_ps)
{
	spll_set_phase_shift(SPLL_ALL_CHANNELS, phase_ps);
	return WR_SPLL_OK;
}

